==> ESCAPE-0.9.4-2893-cp38-cp38-linux_x86_64.whl: escape/utils/widgets/optimizer.py <==
import io

import ipywidgets.widgets as widgets
from escape.utils.widgets.modelstack import MSObjectLayout
from escape.utils.widgets.plot import Plot2dWidget
from escape.utils.widgets.base import lock_update
from IPython.core.display import Javascript, display, HTML


class OptimizerObjectLayout(MSObjectLayout):
    
     
    def __init__(self, escobj, **kwargs):
        modelstack = escobj.modelstack
        self.kwargs=kwargs
        MSObjectLayout.__init__(self, modelstack, **kwargs)
        self.optimizer = escobj
        self.modelstack = modelstack
        self.update()
        escobj.initialization_method = self.on_initialized
        escobj.finalization_method = self.on_finalized
        escobj.iteration_method = self.on_iteration
    
    @lock_update
    def on_iteration(self):
        try:
            self.update_parameter_sliders(silent=True)
            self._update(buf=True)
            self._update_status()
        except Exception as err:
            print(err)
    
    @lock_update
    def on_initialized(self):
        self._update_status()
    
    @lock_update
    def on_finalized(self):
        try:
            self.update_parameter_sliders(silent=True)
            self._update(buf=True)
            self._update_status()
        except Exception as err:
            print(err)
        finally:
            self.startbtn.disabled = False
            self.stopbtn.disabled = True
            self.resetbtn.disabled = False
            self.shakebtn.disabled = False
            self.loadbtn.disabled = False
            self.savebtn.disabled = False
            self.disable_parameters(False)
    
    
    def _update_status(self):
        self.status_label.value = '<b>Best cost:</b> {1:40.4g} <b>Status:</b> {0} '.format(
                self.optimizer.status_msg, self.optimizer.best_cost)
    
    def init_header(self, **kwargs):

        def on_start_click(btn):
            self.startbtn.disabled = True
            self.stopbtn.disabled = False
            self.shakebtn.disabled = True
            self.resetbtn.disabled = True
            self.loadbtn.disabled = True
            self.savebtn.disabled = True
            self.disable_parameters(True)
            self.optimizer(True)
            
        def on_stop_click(btn):
            self.optimizer.stop()
            
        def on_reset_click(btn):
            self.optimizer.reset_to_initial()
            self.update_parameter_sliders(silent=True)
            self._update(buf=False, cost_update = True)
            
        def on_shake_click(btn):
            self.optimizer.shake()
            self.update_parameter_sliders(silent=True)
            self._update(buf=False, cost_update = False)
            
        def on_save_click(btn):
            pars = self.optimizer.save_parameters('')
            js="""
                function download(file, text) {{ 
              
                //creating an invisible element 
                var element = document.createElement('a'); 
                element.setAttribute('href',  
                'data:text/plain;charset=utf-8, ' 
                + encodeURIComponent(text)); 
                element.setAttribute('download', file); 
                document.body.appendChild(element); 
                //onClick property 
                element.click();
                document.body.removeChild(element); 
            }}
            download("parameters.par", "{data}"); 
            """.format(data=pars.replace('\n', '\\n'))
            h = display(Javascript(js), display_id=True)
            h.update(HTML(''))
            
        def on_load_click(btn):
            self.loadbtn._counter = 0
            data = self.loadbtn.data[0]
            self.optimizer.load_parameters(io.StringIO(data.decode('utf-8')))
            self.update_parameter_sliders(silent=True)
            self._update(buf=False, cost_update = True)
        
        btnlay = widgets.Layout( width='40px' )
        
        self.startbtn = widgets.Button(
                description='',
                disabled=False,
                tooltip='Start optimization',
                icon='play',
                layout=btnlay)
        self.startbtn.on_click(on_start_click)
        
        self.stopbtn = widgets.Button(
                description='',
                disabled=True,
                tooltip='Stop optimization',
                icon='stop',
                layout=btnlay)
        self.stopbtn.on_click(on_stop_click)

        
        self.resetbtn = widgets.Button(
                description='',
                disabled=False,
                tooltip='Reset parameters values',
                icon='refresh',
                layout=btnlay)
        
        self.resetbtn.on_click(on_reset_click)
        
        self.shakebtn = widgets.Button(
                description='',
                disabled=False,
                tooltip='Shake parameters values',
                icon='random',
                layout=btnlay)
        
        self.shakebtn.on_click(on_shake_click)
        
        self.savebtn = widgets.Button(
                description='',
                disabled=False,
                tooltip='Save parameters to file',
                icon='save',
                layout=btnlay)
        
        self.loadbtn = widgets.FileUpload(multiple=False, 
                                          icon='folder-open',
                                          description='',
                                          layout=btnlay)
        self.loadbtn.observe(on_load_click, names='value')
        self.savebtn.on_click(on_save_click)
        
        toolbar = widgets.HBox(children=[
            self.startbtn, self.stopbtn,
            self.resetbtn, self.shakebtn, self.loadbtn,
            self.savebtn])
        
        return toolbar
    
    def init_footer(self, **kwargs):
        cost_plot = Plot2dWidget(self, width=850, height=180, xlabel="Iterations", 
                                 ylabel="Cost", tight=True)
        status_label = widgets.HTML('<b>Status: </b>')
        self.status_label = status_label
        self.cost_plot = cost_plot
        return widgets.VBox([status_label, cost_plot])
        
    def _update(self, buf=False, cost_update=True):
        if cost_update:
            costs = self.optimizer.cost_history
            if len(costs):
                self.cost_plot.update(costs[0:,0], costs[0:,1])
        super(OptimizerObjectLayout, self)._update(buf)

==> ESCAPE-0.9.7-3760-cp38-cp38-linux_x86_64.whl: escape/utils/widgets/optimizer.py <==
import io
import threading

import ipywidgets.widgets as widgets
from escape.utils.widgets.modelstack import MSObjectLayout
from escape.utils.widgets.plot import Plot2dWidget
from escape.utils.widgets.base import lock_update
from IPython.core.display import Javascript, display, HTML


class OptimizerObjectLayout(MSObjectLayout):
     
    def __init__(self, escobj, **kwargs):
        modelstack = escobj.modelstack
        self.kwargs = kwargs
        MSObjectLayout.__init__(self, modelstack, **kwargs)
        self.optimizer = escobj
        self.modelstack = modelstack
        self.update()
        self._update()
        self._update_status()
        escobj.initialization_method = self.on_initialized
        escobj.finalization_method = self.on_finalized
        escobj.iteration_method = self.on_iteration
    
    @lock_update
    def on_iteration(self):
        def work():
            try:
                self.update_parameter_sliders(silent=True)
                self._update(buf=True)
                self._update_status()
            except Exception as err:
                print(err)
        #threading.Thread(target=work).start()
        work()
    
    @lock_update
    def on_initialized(self):
        #threading.Thread(target=self._update_status).start()
        self._update_status()
    
    @lock_update
    def on_finalized(self):
        def work():
            try:
                self.update_parameter_sliders(silent=True)
                self._update(buf=True)
                self._update_status()
            except Exception as err:
                print(err)
            finally:
                self.startbtn.disabled = False
                self.stopbtn.disabled = True
                self.resetbtn.disabled = False
                self.shakebtn.disabled = False
                self.loadbtn.disabled = False
                self.savebtn.disabled = False
                self.disable_parameters(False)
        #threading.Thread(target=work).start()
        work()
    
    def _update_status(self):
        self.status_label.value = 'Best cost: {1:40.4g} Status: {0} '.format(
                self.optimizer.status_msg, self.optimizer.best_cost)

    def init_header(self, **kwargs):

        def on_start_click(btn):
            self.startbtn.disabled = True
            self.stopbtn.disabled = False
            self.shakebtn.disabled = True
            self.resetbtn.disabled = True
            self.loadbtn.disabled = True
            self.savebtn.disabled = True
            self.disable_parameters(True)
            threading.Thread(target=self.optimizer, args=(False,)).start()
            
        def on_stop_click(btn):
            self.optimizer.stop()
            
        def on_reset_click(btn):
            self.optimizer.reset_to_initial()
            self.update_parameter_sliders(silent=True)
            self._update(buf=False, cost_update=True)
            
        def on_shake_click(btn):
            self.optimizer.shake()
            self.update_parameter_sliders(silent=True)
            self._update(buf=False, cost_update=False)
            
        def on_save_click(btn):
            pars = self.optimizer.save_parameters('')
            js = """
                function download(file, text) {{ 
              
                //creating an invisible element 
                var element = document.createElement('a'); 
                element.setAttribute('href',  
                'data:text/plain;charset=utf-8, ' 
                + encodeURIComponent(text)); 
                element.setAttribute('download', file); 
                document.body.appendChild(element); 
                //onClick property 
                element.click();
                document.body.removeChild(element); 
            }}
            download("parameters.par", "{data}"); 
            """.format(data=pars.replace('\n', '\\n'))
            h = display(Javascript(js), display_id=True)
            h.update(HTML(''))
            
        def on_load_click(btn):
            self.loadbtn._counter = 0
            data = self.loadbtn.data[0]
            self.optimizer.load_parameters(io.StringIO(data.decode('utf-8')))
            self.update_parameter_sliders(silent=True)
            self._update(buf=False, cost_update=True)
        
        btnlay = widgets.Layout(width='40px')
        
        self.startbtn = widgets.Button(
                description='',
                disabled=False,
                tooltip='Start optimization',
                icon='play',
                layout=btnlay)
        self.startbtn.on_click(on_start_click)
        
        self.stopbtn = widgets.Button(
                description='',
                disabled=True,
                tooltip='Stop optimization',
                icon='stop',
                layout=btnlay)
        self.stopbtn.on_click(on_stop_click)
        
        self.resetbtn = widgets.Button(
                description='',
                disabled=False,
                tooltip='Reset parameters values',
                icon='refresh',
                layout=btnlay)
        
        self.resetbtn.on_click(on_reset_click)
        
        self.shakebtn = widgets.Button(
                description='',
                disabled=False,
                tooltip='Shake parameters values',
                icon='random',
                layout=btnlay)
        
        self.shakebtn.on_click(on_shake_click)
        
        self.savebtn = widgets.Button(
                description='',
                disabled=False,
                tooltip='Save parameters to file',
                icon='save',
                layout=btnlay)
        
        self.loadbtn = widgets.FileUpload(multiple=False,
                                          icon='folder-open',
                                          description='',
                                          layout=btnlay)
        self.loadbtn.observe(on_load_click, names='value')
        self.savebtn.on_click(on_save_click)
        
        toolbar = widgets.HBox(children=[
            self.startbtn, self.stopbtn,
            self.resetbtn, self.shakebtn, 
            #self.loadbtn,
            #self.savebtn
            ])
        
        return toolbar
    
    def init_footer(self, **kwargs):
        cost_plot = Plot2dWidget(self, width=850, height=180, xlabel="Iterations",
                                 ylabel="Cost", tight=True)
        status_label = widgets.HTML('Status: ')
        self.status_label = status_label
        self.cost_plot = cost_plot
        return widgets.VBox([status_label, cost_plot])
        
    def _update(self, buf=False, cost_update=True):
        if cost_update:
            costs = self.optimizer.cost_history
            if len(costs):
                self.cost_plot.update(costs[0:, 0], costs[0:, 1])
        super(OptimizerObjectLayout, self)._update(buf)
